
/**************************************************************************/
/*            ,-.                                                         */
/*           / \  `.  __..-,O           #    #    #  ######   ####        */
/*          :   \ --''_..-'.'           #    ##   #  #       #    #       */
/*          |    . .-' `. '.            #    # #  #  #####   #    #       */
/*          :     .     .`.'            #    #  # #  #       #    #       */
/*           \     `.  /  ..            #    #   ##  #       #    #       */
/*            \      `.   ' .           #    #    #  #        ####        */
/*             `,       `.   \                                            */
/*            ,|,`.        `-.\                                           */
/*           '.||  ``-...__..-`                                           */
/*            |  |                      Stuff you want to know....        */
/*            |__|                                                        */
/*            /||\                                                        */
/*           //||\\                                                       */
/*          // || \\                                                      */
/*       __//__||__\\__                                                   */
/*      '--------------' SSt                                              */
/**************************************************************************/
/* This is a super class used for the basic information composing a mail  */
/* message.  No transport means are available in this class for a purpose */
/* as other classes inherit this class for their specific means of trans- */
/* port.                                                                  */
/* See MailByUNIX.cls, MailBySMTP.cls or MailBySendmail.cls for mail      */
/* transport options.                                                     */
/* Use the interface MailMTA interface to choose which MTA to use.        */
/**************************************************************************/


class com.amduus.mail.Mail:

  define public variable ToEmail as character no-undo.
  define public variable CCEmail as character no-undo.
  define public variable BCCEmail as character no-undo.
  define public variable FromEmail as character no-undo.
  define public variable Subject as character no-undo.
  define public variable HTMLBody as longchar no-undo.
  define public variable TextBody as longchar no-undo.
  
  /* Gives you the option of where to put temp files while constructing mail */
  
  define public variable TempDirectory as character init "/tmp/" no-undo.
  
  /* Gives you the option to delete the file in the temporary directory */
  
  define public variable DoDelete as logical init true no-undo.
  
  /* Gives you the option of discovering the temp file that was created on execution */
  
  define public variable TempMailFile as character no-undo.  
  
  define protected temp-table Attachment
    field CID as character           /* for CID attachments */
    field FileOnDisk as character    /* Where the file is on disk */
    field FileOnMessage as character /* What to name the file on the message */
    field MimeType as character      /* User can determine the mimetype */
    .
    
  define protected variable Strings as com.amduus.misc.Strings no-undo.
  
  /**************************************************************************/
  /* This class needs the Strings tools so make it when we instance.        */
  /**************************************************************************/
  
  constructor public Mail():

    Strings = new com.amduus.misc.Strings().
        
  end. /* constructor */
  
  destructor public Mail():
  
    delete object Strings.
    
  end. /* destructor */
    
  /**************************************************************************/
  /* CIDs are files embedded in HTML messages usually some kind of graphic. */
  /* In the HTML, one refers to them by their CID id instead of by a URL.   */
  /* If the mimetype is ? it will be figured out automatically.             */
  /**************************************************************************/
  
  method public character AttachCID (input CID as character, input FileOnDisk as character, input MimeType as character):
  
    create Attachment.
    
    Attachment.CID = CID.
    Attachment.FileOnDisk = FileOnDisk.
    Attachment.MimeType = MimeType.
  
    
  end. /* AttachCID*/
  
  /**************************************************************************/
  /* Provide a means to delete a CID from the message.                      */
  /**************************************************************************/
  
  method public logical DeleteCID(input CID as character):
  
    define variable Success as logical init false no-undo.
    
    for each Attachment exclusive-lock
    where Attachment.CID = CID:
    
      delete Attachment.
      Success = true.
      
    end. /* for each */
    
    return Success.
    
  end. /* DeleteCID */
  
  /**************************************************************************/
  /* One can use this to attach (multiple) files to the email message which */
  /* are named in the email as FileOnMessage.                               */
  /* If the mimetype is ? it will be figured out automatically.             */
  /**************************************************************************/
  
  method public void AttachFile (input FileOnDisk as character, input FileOnMessage as character, input MimeType as character):
  
    create Attachment.
    
    Attachment.FileOnDisk = FileOnDisk.
    Attachment.FileOnMessage = FileOnMessage.
    Attachment.MimeType  = MimeType.
  
  end. /* AttachFile */
  
  /**************************************************************************/
  /* Provide ability to delete file from the message.                       */
  /**************************************************************************/
  
  method public logical DeleteFile (input FileOnMessage as character):
  
    define variable Success as logical init false no-undo.
    
    for each Attachment exclusive-lock
    where Attachment.FileOnMessage = FileOnMessage:
    
      delete Attachment.
      Success = true.
      
    end. /* for each */
    
    return Success.  
  
  end. /* DeleteFile */
  
  /**************************************************************************/
  /* Used for debugging.                                                    */
  /**************************************************************************/
  
  method public void ExportAttachments (input ExportFileName as character):
  
    output to value (ExportFileName).
    
    for each Attachment no-lock:
            
      export Attachment.
    
    end.  /* for each Attachment */
    
    output close.
    
  end. /* ExportAttachments */
  
  /**************************************************************************/
  /* Generate a text message.  RFC822 compliant.  MTA independent.          */
  /**************************************************************************/
  
  method protected void GenerateTextMessage(input DestinationFileName as character):
  
    output to value (DestinationFileName).
   
    /* Put out headers of the message */
    
    put unformatted "To: " ToEmail Strings:CRLF.
    put unformatted "Cc: " CCEmail Strings:CRLF.
    put unformatted "Subject: " Subject Strings:CRLF.  
    put unformatted "Content-Type: text/plain" Strings:CRLF.
    put unformatted Strings:CRLF.
    
    /* The body is a longchar so we need to chunk that along */
    
    Strings:NewChunk(TextBody).
    Strings:ChunkSize = 1000.
    
    do while not Strings:EndOfChunkData():
      put unformatted Strings:GetChunk().
    end. /* do while not Strings:IsLastChunk() */    
    
    put unformatted Strings:CRLF.
    
    /* Ready to go! */
    
    output close.
  
  end. /* GenerateTextMessage */
  
  /**************************************************************************/
  /* Generate MIME message.  RFC2046 compliant.  MTA independent.           */
  /**************************************************************************/
  
  method protected void GenerateMimeMessage (input DestinationFileName as character):
  
    define variable Boundary as character no-undo.

    define variable FileEncoder as com.amduus.mail.FileEncoder no-undo.
    
    FileEncoder = new com.amduus.mail.FileEncoder().
  
    Boundary = "amduusgc0p4Jq0M2Yt08j34c0p".
    
    output to value (DestinationFileName).
   
    /* Put out headers of the message */
    
    put unformatted "To: " ToEmail Strings:CRLF.
    put unformatted "Cc: " CCEmail Strings:CRLF.
    put unformatted "Subject: " Subject Strings:CRLF.
    
    /* Whether we are text or HTML, we always are MIME */
    /* If we want only text, we will make a IsTextOnly flag and process */
    /* according to that.                                               */
    /* Each entry in the boundary meets RFC2045 and RFC822 compliant.   */
    
    put unformatted "MIME-Version: 1.0" Strings:CRLF.
    put unformatted "Content-Type: multipart/mixed; boundary=" Boundary Strings:CRLF.
    
    put unformatted Strings:CRLF.
    
    put unformatted "This is a MIME message.  You need a Mime email reader to make sense of this." Strings:CRLF.
    
    put unformatted Strings:CRLF.    
    
    /* First we toss out any text portion of the message (as some mail */
    /* readers are cool with text or html based on a configuration.)   */
    
    if TextBody <> "" then do:

      put unformatted "--" Boundary Strings:CRLF.
      put unformatted "Content-type: text/plain" Strings:CRLF.
      put unformatted Strings:CRLF.
    
      Strings:NewChunk(TextBody).
      Strings:ChunkSize = 1000.
    
      do while not Strings:EndOfChunkData():
        put unformatted Strings:GetChunk().
      end. /* do while not Strings:IsLastChunk() */
    
      put unformatted Strings:CRLF.
      put unformatted Strings:CRLF.
    
    end. /* if TextBody <> "" */
    
    /* Next is the HTML portion. */
    
    if HTMLBody <> "" then do:
      
      put unformatted "--" Boundary Strings:CRLF.
      put unformatted "Content-type: text/html" Strings:CRLF.
      put unformatted Strings:CRLF.
      
      Strings:NewChunk(HTMLBody).
      Strings:ChunkSize = 1000.
      
      do while not Strings:EndOfChunkData():
        put unformatted Strings:GetChunk().
      end. /* do while not Strings:IsLastChunk() */
      
      put unformatted Strings:CRLF. 
      put unformatted Strings:CRLF.
        
    end. /* if HTMLBody <> "" */
    
    /* Next we do any attachments */
    
    for each Attachment no-lock
      where Attachment.CID = "":

      FileEncoder:EncodeFile(Attachment.FileOnDisk).
      if Attachment.MimeType = "" then Attachment.MimeType = FileEncoder:DetermineMimeType().
      
      put unformatted "--" Boundary Strings:CRLF.
      put unformatted "Content-type: " Attachment.MimeType Strings:CRLF.
      put unformatted "Content-Transfer-Encoding: base64" Strings:CRLF.
      put unformatted "Content-disposition: attachment; filename=" Attachment.FileOnMessage Strings:CRLF.
      put unformatted Strings:CRLF.    
      

      Strings:NewChunk(FileEncoder:FileEncodedData).
      Strings:ChunkSize = 1000.
    
      do while not Strings:EndOfChunkData():
        put unformatted Strings:GetChunk().
      end. /* do while not Strings:IsLastChunk() */

      put unformatted Strings:CRLF. 
      
    end. /* for each Attachment */    
    
    /* Next we do any CID attachments */
    
    for each Attachment no-lock
      where Attachment.CID <> "":

      FileEncoder:EncodeFile(Attachment.FileOnDisk).
      if Attachment.MimeType = "" then Attachment.MimeType = FileEncoder:DetermineMimeType().
    
      put unformatted "--" Boundary Strings:CRLF.
      put unformatted "Content-type: " Attachment.MimeType Strings:CRLF.
      put unformatted "Content-Transfer-Encoding: base64" Strings:CRLF.
      put unformatted "Content-Id: " Attachment.CID Strings:CRLF.      
      put unformatted Strings:CRLF.    

      Strings:NewChunk(FileEncoder:FileEncodedData).
      Strings:ChunkSize = 1000.
    
      do while not Strings:EndOfChunkData():
        put unformatted Strings:GetChunk().
      end. /* do while not Strings:IsLastChunk() */
      
      put unformatted Strings:CRLF. 
      
    end. /* for each Attachment */
    
    /* Done deal, send it on it's way! */
    
    put unformatted "--" Boundary "--" Strings:CRLF.
    
    output close.  
    
    delete object FileEncoder.
  
  end. /* GenerateMimeMessage */
  
end. /* class */

/**************************************************************************
                                UNIT TEST CODE
                                --------------
                                
                                Prepare a text message
                               
define variable M as com.amduus.mail.Mail no-undo.

M = new com.amduus.mail.Mail().

M:ToEmail = "sauge@localhost".
M:FromEmail = "sauge@localhost".
M:Subject = "Test 1".
M:TextBody = "Here is test 1".

M:GenerateTextMessage("/tmp/a.1").

delete object M.


**************************************************************************/
