
/**************************************************************************/
/*            ,-.                                                         */
/*           / \  `.  __..-,O           #    #    #  ######   ####        */
/*          :   \ --''_..-'.'           #    ##   #  #       #    #       */
/*          |    . .-' `. '.            #    # #  #  #####   #    #       */
/*          :     .     .`.'            #    #  # #  #       #    #       */
/*           \     `.  /  ..            #    #   ##  #       #    #       */
/*            \      `.   ' .           #    #    #  #        ####        */
/*             `,       `.   \                                            */
/*            ,|,`.        `-.\                                           */
/*           '.||  ``-...__..-`                                           */
/*            |  |                      Stuff you want to know....        */
/*            |__|                                                        */
/*            /||\                                                        */
/*           //||\\                                                       */
/*          // || \\                                                      */
/*       __//__||__\\__                                                   */
/*      '--------------' SSt                                              */
/**************************************************************************/
/* This is a mail transport.  It uses the /usr/bin/mail command to send   */
/* See Mail.cls for information about generating mail messages.           */
/* out mail.                                                              */
/* This has been tested on Linux.                                         */
/**************************************************************************/

/**************************************************************************/
/*                                                                        */
/*                  BEWARE!  THAR BE SEA-MONSTERS HERE!                   */
/*                                                     _  _               */
/*                                                    ' \/ '              */
/*    _  _                        <|                                      */
/*     \/              __'__     __'__      __'__                         */
/*                    /    /    /    /     /    /                         */
/*                   /\____\    \____\     \____\               _  _      */
/*                  / ___!___   ___!___    ___!___               \/       */
/*                // (      (  (      (   (      (                        */
/*              / /   \______\  \______\   \______\                       */
/*            /  /   ____!_____ ___!______ ____!_____                     */
/*          /   /   /         //         //         /                     */
/*       /    /   |         ||         ||         |                       */
/*      /_____/     \         \\         \\         \                     */
/*            \      \_________\\_________\\_________\                    */
/*             \         |          |         |                           */
/*              \________!__________!_________!________/                  */
/*               \|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_/|                  */
/*                \    _______________                /                   */
/* ^^^%%%^%^^^%^%%^\_"/_)/_)_/_)__)/_)/)/)_)_"_'_"_//)/)/)/)%%%^^^%^^%%%%^*/
/* ^!!^^"!%%!^^^!^^^!!^^^%%%%%!!!!^^^%%^^^!!%%%%^^^!!!!!!%%%^^^^%^^%%%^^^!*/
/*                                                                        */
/*                                                                        */
/**************************************************************************/
/* The /usr/bin/mail command works differently on different flavor's of   */
/* UNIX.  This may not be the best MTA program to use.  Suggest using the */
/* MailBySendmail.cls class to use mail.                                  */
/**************************************************************************/

class com.amduus.mail.MailByUNIX inherits com.amduus.mail.Mail:

  define private variable CLI as com.amduus.security.CLI no-undo.
  
  /**************************************************************************/
  /* Constructor                                                            */
  /**************************************************************************/
  
  constructor public MailByUNIX():

    super().  
    
    /* Since it is command line based, we need these tools */
    
    CLI = new com.amduus.security.CLI().

  end. /* constructor */
  
  /**************************************************************************/
  /* Destructor.                                                            */
  /**************************************************************************/
  
  destructor public MailByUNIX():

    delete object CLI.
    
  end. /* desctructor */
  
  /**************************************************************************/
  /* Sends the email using the mail command.  Text only - no mime.          */
  /**************************************************************************/
  
  method public void SendTextEMail():
  
    define variable CmdLine as character no-undo.

    
    /* Since this is a command line tool we are using, we escape the data */
    
    ToEmail = CLI:EscapeCLI(ToEmail).
    CCEmail = CLI:EscapeCLI(CCEmail).
    BCCEmail = CLI:EscapeCLI(BCCEmail).
    
    /* We assemble the message in a temporary file and send it along */
    
    TempMailFile = TempDirectory + Strings:RandomString(10) + ".mail".
   
    GenerateTextMessage(TempMailFile).
    
    /* This comes with the mail invocation for Linux mail. */
    /* Solaris will probably not want the -t               */
    
    CmdLine = "/usr/bin/mail -t "  
            + ToEmail + " " 
            + CCEmail + " " 
            + BCCEmail + " "
            + "< " + TempMailFile
            .
            
    /* Webspeed needs an output destination for os-command */
    
    output to /dev/null.
    os-command value (CmdLine).
    output close.
    
    /* Housekeeping */
    
    if DoDelete then os-delete value(TempMailFile).
    
  end. /* SendMail */
    
  /**************************************************************************/
  /* Uses information in Mail.cls to generate a MIME compliant email.       */
  /**************************************************************************/
  
  method public void SendMimeEMail():
  
    define variable CmdLine as character no-undo.

    
    /* Since this is a command line tool we are using, we escape the data */
    
    ToEmail = CLI:EscapeCLI(ToEmail).
    CCEmail = CLI:EscapeCLI(CCEmail).
    BCCEmail = CLI:EscapeCLI(BCCEmail).
    
    /* We assemble the message in a temporary file and send it along */
    
    TempMailFile = TempDirectory + Strings:RandomString(10) + ".mail".
   
    GenerateMimeMessage(TempMailFile).
    
    CmdLine = "mail -t "
            + ToEmail + " " 
            + CCEmail + " " 
            + BCCEmail + " "
            + "< " + TempMailFile
            .
            
    /* Webspeed needs an output destination for os-command */
    
    output to /dev/null.
    os-command value (CmdLine).
    output close.
    
    /* Housekeeping */
    
    if DoDelete then os-delete value(TempMailFile).
    
  end. /* SendMimeMail */

end. /* class */

/**************************************************************************

UNIT TEST

define variable M as com.amduus.mail.MailByUNIX no-undo.

M = new com.amduus.mail.MailByUNIX().

M:ToEmail = "sauge@localhost".
M:FromEmail = "root@localhost".
M:Subject = "Test 1".
M:TextBody = "This is a test email".

M:SendTextEMail ().

delete object M.

**************************************************************************/
